@@ -2,7 +2,7 @@
 /*
  * This file is part of maemo-af-desktop
  *
- * Copyright (C) 2006 Nokia Corporation.
+ * Copyright (C) 2006, 2007 Nokia Corporation.
  *
  * Contact: Karoliina Salminen <karoliina.t.salminen@nokia.com>
  *
@@ -29,43 +29,26 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
-#include <gtk/gtk.h>
-#include <locale.h>
-#include <libintl.h>
-#include <math.h>
 #include <stdio.h>
 #include <string.h>
 
 #include <libosso.h>
 #include <glib.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-note.h>
 #include <hildon-widgets/hildon-defines.h>
-#include <hildon-widgets/hildon-caption.h>
-#include <hildon-widgets/hildon-color-button.h>
 
-/*#include "background-manager.h"*/
+#include <libhildonwm/hd-wm.h>
 
-#include "hildon-home-l10n.h"
 #include "hildon-home-area.h"
 #include "hildon-home-applet.h"
 #include "hildon-home-titlebar.h"
 #include "hildon-home-window.h"
-#include "hildon-home-background-dialog.h"
 
-#define HILDON_HOME_LEFT_PADDING	80	/* keep synced with the TN width */
-#define HILDON_HOME_WINDOW_WIDTH	800
-#define HILDON_HOME_WINDOW_HEIGHT	480
-#define HILDON_HOME_TOP_BAR_HEIGHT	60
-
-#define LAYOUT_OPENING_BANNER_TIMEOUT 2500
-
-#define HH_AREA_CONFIGURATION_FILE      ".osso/hildon-desktop/hildon-home-layout.conf"
-#define HH_AREA_GLOBAL_CONFIGURATION_FILE "/etc/hildon-desktop/hildon-home-layout.conf"
 
+#define HH_AREA_CONFIGURATION_FILE        ".osso/hildon-desktop/home-layout.conf"
+#define HH_AREA_GLOBAL_CONFIGURATION_FILE "/etc/hildon-desktop/home-layout.conf"
 
 /* DBUS defines */
 #define STATUSBAR_SERVICE_NAME          "statusbar"
@@ -81,227 +64,39 @@
 #define TASKNAV_INSENSITIVE_METHOD      "tasknav_insensitive"
 #define TASKNAV_SENSITIVE_METHOD        "tasknav_sensitive"
 
-/* Style properties */
-#define HH_BLEND_IMAGE_SIDEBAR_NAME     "HildonHomeLeftEdge"
-#define HH_BLEND_IMAGE_TITLEBAR_NAME    "HildonHomeTitleBar"
-
 #define HILDON_HOME_WINDOW_GET_PRIVATE(obj) \
 (G_TYPE_INSTANCE_GET_PRIVATE ((obj), HILDON_TYPE_HOME_WINDOW, HildonHomeWindowPrivate))
 
 struct _HildonHomeWindowPrivate
 {
-/*  BackgroundManager *bg_manager;*/
+  GtkWidget    *titlebar;
+  GtkWidget    *applet_area;
 
-  GtkWidget *align;
-  
-  GtkWidget *titlebar;
-  GtkWidget *applet_area;
-  GtkWidget *main_area;
+  GtkWidget    *layout_mode_banner;
+  guint         layout_mode_banner_to;
 
-  GtkWidget *layout_mode_banner;
-  guint      layout_mode_banner_to;
+  GdkRectangle *work_area;
 
-  guint is_dimmed : 1;
-  guint is_lowmem : 1;
-  guint is_inactive : 1;
+  guint         is_dimmed : 1;
+  guint         is_lowmem : 1;
+  guint         is_inactive : 1;
 
-  guint selecting_applets : 1;
+  guint         selecting_applets : 1;
 };
 
 G_DEFINE_TYPE (HildonHomeWindow, hildon_home_window, HILDON_DESKTOP_TYPE_WINDOW);
 
+static void
+hildon_home_window_set_work_area (HildonHomeWindow *window,
+                                  GdkRectangle *workarea);
+
 enum
 {
   PROP_0,
 
-  PROP_OSSO_CONTEXT
+  PROP_WORK_AREA
 };
 
-static gboolean
-destroy_banner (HildonHomeWindow *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-
-  if (priv->layout_mode_banner)
-    {
-      gtk_widget_destroy (priv->layout_mode_banner);
-      priv->layout_mode_banner = NULL;
-      g_signal_handlers_disconnect_by_func (G_OBJECT (priv->applet_area),
-                                            G_CALLBACK (destroy_banner),
-                                            window);
-    }
-  priv->layout_mode_banner_to = 0;
-
-  return FALSE;
-}
-
-static void
-hildon_home_window_show_layout_mode_banner (HildonHomeWindow *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-  
-  if (!priv->layout_mode_banner)
-    {
-      priv->layout_mode_banner = 
-          hildon_banner_show_animation (GTK_WIDGET (window),
-                                        NULL,
-                                        HH_LAYOUT_MODE_BANNER);
-      priv->layout_mode_banner_to = 
-          g_timeout_add (LAYOUT_OPENING_BANNER_TIMEOUT,
-                         (GSourceFunc)destroy_banner,
-                         window);
-
-      g_signal_connect_swapped (G_OBJECT (priv->applet_area),
-                                "applet-change-start",
-                                G_CALLBACK (destroy_banner),
-                                window);
-
-
-    }
-    
-}
-
-static void
-titlebar_select_applets_activate_cb (HildonHomeTitlebar *titlebar,
-                                     HildonHomeWindow   *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-  
-  priv->selecting_applets = TRUE;
-  g_debug ("Emitting select-plugins");
-  g_signal_emit_by_name (window, "select-plugins", NULL);
-  g_debug ("After select-plugins ");
-  priv->selecting_applets = FALSE;
-
-  if (HILDON_IS_HOME_AREA (priv->applet_area))
-    {
-      HildonHomeArea *area = HILDON_HOME_AREA (priv->applet_area);
-
-      /* If we are in layout mode, we only save when layout mode is
-       * accepted */
-      if (!hildon_home_area_get_layout_mode (area))
-        g_signal_emit_by_name (window, "save", NULL);
-    }
-}
-
-static void
-titlebar_set_background_activate_cb (HildonHomeTitlebar *titlebar,
-                                     HildonHomeWindow   *window)
-{
-  /*home_bgd_dialog_run (GTK_WINDOW (window));*/
-}
-
-static void
-titlebar_layout_mode_activate_cb (HildonHomeTitlebar *titlebar,
-				  HildonHomeWindow   *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-  g_debug ("layout mode activate");
-  
-  if (priv->is_lowmem)
-    {
-      hildon_banner_show_information (NULL,
-                                      NULL,
-                                      HH_LOW_MEMORY_TEXT);
-      return;
-    }
-
-  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
-                                    TRUE);
-
-}
-
-static void
-titlebar_layout_accept_cb (HildonHomeTitlebar *titlebar,
-                           HildonHomeWindow   *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-  gchar *filename;
-  g_debug ("layout accepted");
-  
-  if (hildon_home_area_get_overlaps (HILDON_HOME_AREA (priv->applet_area)))
-    {
-      hildon_home_window_show_information_note (window,
-                                                HH_LAYOUT_OVERLAP_TEXT);
-      return;
-    }
-  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
-                                    FALSE);
-  
-  filename = g_build_filename (g_getenv ("HOME"),
-                               HH_AREA_CONFIGURATION_FILE,
-                               NULL);
-  
-  if (hildon_home_area_save_configuration (HILDON_HOME_AREA (priv->applet_area),
-                                       filename) < 0)
-    hildon_home_window_show_information_note (window,
-                                              HH_FLASH_FULL_TEXT); 
-
-  g_free (filename);
-
-  g_signal_emit_by_name (window, "save", NULL);
-}
-
-static void
-titlebar_layout_cancel_cb (HildonHomeTitlebar *titlebar,
-                           HildonHomeWindow   *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-  g_debug ("layout discarded");
-
-  if (hildon_home_area_get_layout_changed 
-      (HILDON_HOME_AREA (priv->applet_area)))
-    {
-      GtkWidget *note;
-      gint response;
-      gchar *user_filename = NULL; 
-      gchar *filename = NULL;
-
-      note = hildon_note_new_confirmation_add_buttons 
-          (NULL,
-           HH_LAYOUT_CANCEL_TEXT,
-           HH_LAYOUT_CANCEL_YES,
-           GTK_RESPONSE_ACCEPT,
-           HH_LAYOUT_CANCEL_NO,
-           GTK_RESPONSE_CANCEL,
-           NULL);
-
-      response = gtk_dialog_run (GTK_DIALOG (note));
-      gtk_widget_destroy (note);
-
-      if (response != GTK_RESPONSE_ACCEPT)
-        return;
-  
-      user_filename = g_build_filename (g_getenv ("HOME"),
-                                        HH_AREA_CONFIGURATION_FILE,
-                                        NULL);
-
-      if (g_file_test (user_filename, G_FILE_TEST_EXISTS))
-        filename = user_filename;
-      else
-        filename = HH_AREA_GLOBAL_CONFIGURATION_FILE;
-
-      hildon_home_area_load_configuration (HILDON_HOME_AREA (priv->applet_area),
-                                           filename);
-
-      g_free (user_filename);
-
-      g_signal_emit_by_name (window, "load", NULL);
-    }
-
-  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
-                                    FALSE);
-}
-
-static void
-area_layout_mode_start (HildonHomeArea *area,
-                        HildonHomeWindow   *window)
-{
-  hildon_home_window_set_desktop_dimmed (window, TRUE);
-
-  hildon_home_window_show_layout_mode_banner (window);
-}
-
 static void
 area_layout_mode_started (HildonHomeArea *area,
                           HildonHomeWindow   *window)
@@ -324,15 +119,6 @@ area_layout_mode_ended (HildonHomeArea *area,
 
   hildon_home_window_set_desktop_dimmed (window, FALSE);
 
-  if (priv->layout_mode_banner_to)
-    {
-      g_source_remove (priv->layout_mode_banner_to);
-      if (priv->layout_mode_banner)
-        {
-          gtk_widget_destroy (priv->layout_mode_banner);
-          priv->layout_mode_banner = NULL;
-        }
-    }
 }
 
 static void
@@ -341,7 +127,6 @@ area_add (HildonHomeArea   *area,
           HildonHomeWindow *window)
 {
   HildonHomeWindowPrivate *priv = window->priv;
-  g_signal_emit_by_name (priv->titlebar, "applet-added", area);
 
   if (HILDON_IS_HOME_AREA (priv->applet_area))
     {
@@ -368,100 +153,30 @@ area_remove (HildonHomeArea   *area,
              HildonHomeWindow *window)
 {
   HildonHomeWindowPrivate *priv = window->priv;
-  g_signal_emit_by_name (priv->titlebar, "applet-removed", area);
   
   /* Only save if not in layout mode (if we just removed an applet). If in
    * layout mode it will be saved if the layout is accepted */
   if (!hildon_home_area_get_layout_mode (HILDON_HOME_AREA (priv->applet_area)))
     {
+      GError *error = NULL;
       gchar *filename = g_build_filename (g_getenv ("HOME"),
                                           HH_AREA_CONFIGURATION_FILE,
                                           NULL);
 
-      if (hildon_home_area_save_configuration (
-                                           HILDON_HOME_AREA (priv->applet_area),
-                                           filename) < 0 )
-        hildon_home_window_show_information_note (window,
-                                                 HH_FLASH_FULL_TEXT);
+      hildon_home_area_save_configuration (HILDON_HOME_AREA (priv->applet_area),
+                                           filename,
+                                           &error);
+
+      if (error)
+        {
+          g_signal_emit_by_name (window, "io-error", error);
+          g_error_free (error);
+        }
 
       g_free (filename);
     }
 }
 
-#if 0
-static void
-background_manager_load_complete_cb (BackgroundManager *manager,
-                                     HildonHomeWindow  *window)
-{
-  g_debug (G_STRLOC ": load complete");
-}
-
-static void
-background_manager_load_cancel_cb (BackgroundManager *manager,
-                                   HildonHomeWindow  *window)
-{
-  g_debug (G_STRLOC ": load cancelled");
-}
-
-static void
-background_manager_load_error_cb (BackgroundManager *manager,
-                                  const GError      *error,
-                                  HildonHomeWindow  *window)
-{
-  const gchar *text;
-  
-  g_return_if_fail (error);
-  
-  if (error->domain == background_manager_error_quark ())
-    {
-      switch (error->code)
-        {
-          case BACKGROUND_MANAGER_ERROR_IO:
-              text = HH_NO_CONNECTION_TEXT;
-              break;
-          case BACKGROUND_MANAGER_ERROR_CORRUPT:
-              text = HH_FILE_CORRUPTED_TEXT;
-              break;
-          case BACKGROUND_MANAGER_ERROR_MMC_OPEN:
-              text = HH_MMC_OPEN_TEXT;
-              break;
-          case BACKGROUND_MANAGER_ERROR_MEMORY:
-          default:
-              text = NULL;
-              break;
-        }
-    }
-  else
-    text = NULL;
-
-  if (text)
-    hildon_home_window_show_information_note (window, text);
-}
-
-static void
-background_manager_changed_cb (BackgroundManager *manager,
-			       GdkPixbuf         *pixbuf,
-			       HildonHomeWindow  *window)
-{
-  HildonHomeWindowPrivate *priv = window->priv;
-
-  if (!pixbuf)
-    return;
-  else
-    {
-      priv = window->priv;
-      
-      /* force the event boxes to be window-less and avoid covering
-       * the area we paint directly onto
-       */
-      gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->titlebar), FALSE);
-      gtk_event_box_set_visible_window (GTK_EVENT_BOX (priv->main_area), FALSE);
-      
-      gtk_widget_queue_draw (GTK_WIDGET (window));
-    }
-}
-#endif
-
 static void
 hildon_home_window_lowmem (HildonHomeWindow   *window,
                            gboolean            is_lowmem)
@@ -492,6 +207,7 @@ hildon_home_window_background (HildonHomeWindow   *window,
         {
           gchar *user_filename = NULL; 
           gchar *filename = NULL;
+          GError *error = NULL;
 
           hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
                                             FALSE);
@@ -506,7 +222,14 @@ hildon_home_window_background (HildonHomeWindow   *window,
             filename = HH_AREA_GLOBAL_CONFIGURATION_FILE;
 
           hildon_home_area_load_configuration (HILDON_HOME_AREA (priv->applet_area),
-                                               filename);
+                                               filename,
+                                               &error);
+
+          if (error)
+            {
+              g_signal_emit_by_name (window, "io-error", error);
+              g_error_free (error);
+            }
 
           g_free (user_filename);
         }
@@ -532,101 +255,84 @@ hildon_home_window_system_inactivity (HildonHomeWindow   *window,
                          (gpointer)is_inactive);
 }
 
-static const gchar *
-get_sidebar_image_from_theme (GtkWidget *widget)
+static void
+hildon_home_window_layout_mode_accept (HildonHomeWindow *window)
 {
-  GtkStyle *style;
+  HildonHomeWindowPrivate *priv = window->priv;
+  gchar *filename;
+  GError *error = NULL;
   
-  style = gtk_rc_get_style_by_paths (gtk_widget_get_settings (widget),
-                                     HH_BLEND_IMAGE_SIDEBAR_NAME,
-                                     NULL,
-                                     G_TYPE_NONE);
-  
-  if (style && style->rc_style->bg_pixmap_name[0])
-    return style->rc_style->bg_pixmap_name[0];
+  if (hildon_home_area_get_overlaps (HILDON_HOME_AREA (priv->applet_area)))
+    return;
 
-  return NULL;
+  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
+                                    FALSE);
+  
+  filename = g_build_filename (g_getenv ("HOME"),
+                               HH_AREA_CONFIGURATION_FILE,
+                               NULL);
+  
+  hildon_home_area_save_configuration (HILDON_HOME_AREA (priv->applet_area),
+                                       filename,
+                                       &error);
+
+  if (error)
+    {
+      g_signal_emit_by_name (window,
+                             "io-error",
+                             error);
+      g_error_free (error);
+    }
+
+  g_free (filename);
 }
 
-
-static const gchar *
-get_titlebar_image_from_theme (GtkWidget *widget)
+static void
+hildon_home_window_layout_mode_cancel (HildonHomeWindow *window)
 {
-  GtkStyle *style;
+  HildonHomeWindowPrivate *priv = window->priv;
+  gchar *user_filename = NULL;
+  gchar *filename = NULL;
+  GError *error = NULL;
 
-  style = gtk_rc_get_style_by_paths (gtk_widget_get_settings (widget),
-                                     HH_BLEND_IMAGE_TITLEBAR_NAME,
-                                     NULL,
-                                     G_TYPE_NONE);
+  user_filename = g_build_filename (g_getenv ("HOME"),
+                                    HH_AREA_CONFIGURATION_FILE,
+                                    NULL);
 
-  if (style && style->rc_style->bg_pixmap_name[0])
-    return style->rc_style->bg_pixmap_name[0];
+  if (g_file_test (user_filename, G_FILE_TEST_EXISTS))
+    filename = user_filename;
+  else
+    filename = HH_AREA_GLOBAL_CONFIGURATION_FILE;
 
-  return NULL;
+  hildon_home_area_load_configuration (HILDON_HOME_AREA (priv->applet_area),
+                                       filename,
+                                       &error);
+
+  if (error)
+    {
+      g_signal_emit_by_name (window, "io-error", error);
+      g_error_free (error);
+    }
+
+  g_free (user_filename);
+
+  g_signal_emit_by_name (window, "load", NULL);
+
+  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
+                                    FALSE);
 }
 
-#if 0
 static void
 hildon_home_window_show (GtkWidget *widget)
 {
   gtk_widget_realize (widget);
-  gdk_window_set_type_hint (widget->window,
-		            GDK_WINDOW_TYPE_HINT_DESKTOP);
 
-  background_manager_set_desktop (background_manager_get_default (),
-                                  widget->window);
+  gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
+  gdk_window_set_type_hint (widget->window,
+                            GDK_WINDOW_TYPE_HINT_DESKTOP);
   
   GTK_WIDGET_CLASS (hildon_home_window_parent_class)->show (widget);
 }
-#endif
-
-static void
-hildon_home_window_style_set (GtkWidget *widget,
-			      GtkStyle  *old_style)
-{
-/*  HildonHomeWindowPrivate *priv = HILDON_HOME_WINDOW (widget)->priv;*/
-  static gchar *current_titlebar_image;
-  static gchar *current_sidebar_image;
-  const gchar *new_titlebar_image, *new_sidebar_image;
-
-  if (GTK_WIDGET_CLASS (hildon_home_window_parent_class)->style_set)
-    GTK_WIDGET_CLASS (hildon_home_window_parent_class)->style_set (widget,
-		    						   old_style);
-  
-  new_titlebar_image = get_titlebar_image_from_theme (widget);
-  new_sidebar_image = get_sidebar_image_from_theme (widget);
-
-  /* avoid resetting the background when the window is exposed for the
-   * first time
-   */
-  if (!old_style)
-    {
-      current_titlebar_image = g_strdup (new_titlebar_image);
-      current_sidebar_image = g_strdup (new_sidebar_image);
-      return;
-    }
-
-  /* avoid resetting the background if the theme hasn't changed */
-  if (g_str_equal (current_titlebar_image, new_titlebar_image) &&
-      g_str_equal (current_sidebar_image, new_sidebar_image)/* &&
-      background_manager_refresh_from_cache (priv->bg_manager)*/)
-    {
-      return;
-    }
-
-  g_free(current_titlebar_image);
-  current_titlebar_image = g_strdup (new_titlebar_image);
-  
-  g_free(current_sidebar_image);
-  current_sidebar_image = g_strdup (new_sidebar_image);
-
-#if 0
-  background_manager_set_components (priv->bg_manager,
-                                     new_titlebar_image,
-                                     new_sidebar_image,
-                                     TRUE);
-#endif
-}
 
 static gboolean
 hildon_home_window_key_press_event (GtkWidget *widget,
@@ -662,42 +368,47 @@ hildon_home_window_key_press_event (GtkWidget *widget,
 static void
 hildon_home_window_finalize (GObject *gobject)
 {
-#if 0
-  HildonHomeWindowPrivate *priv = HILDON_HOME_WINDOW (gobject)->priv;
-
-  g_signal_handlers_disconnect_by_func (priv->bg_manager,
-		                        background_manager_changed_cb,
-					HILDON_HOME_WINDOW (gobject));
-#endif
-  
   G_OBJECT_CLASS (hildon_home_window_parent_class)->finalize (gobject);
 }
 
 static void
 hildon_home_window_set_property (GObject      *gobject,
-				 guint         prop_id,
-				 const GValue *value,
-				 GParamSpec   *pspec)
+                                 guint         prop_id,
+                                 const GValue *value,
+                                 GParamSpec   *pspec)
 {
   switch (prop_id)
     {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
-      break;
+      case PROP_WORK_AREA:
+          hildon_home_window_set_work_area (HILDON_HOME_WINDOW (gobject),
+                                            (GdkRectangle *)
+                                              g_value_get_pointer (value));
+          break;
+      default:
+          G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+          break;
     }
 }
 
 static void
 hildon_home_window_get_property (GObject    *gobject,
-				 guint       prop_id,
-				 GValue     *value,
-				 GParamSpec *pspec)
+                                 guint       prop_id,
+                                 GValue     *value,
+                                 GParamSpec *pspec)
 {
+  HildonHomeWindowPrivate *priv;
+ 
+  g_return_if_fail (HILDON_IS_HOME_WINDOW (gobject));
+  priv = HILDON_HOME_WINDOW (gobject)->priv;
+
   switch (prop_id)
     {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
-      break;
+      case PROP_WORK_AREA:
+          g_value_set_pointer (value, priv->work_area);
+          break;
+      default:
+          G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+          break;
     }
 }
 
@@ -707,6 +418,7 @@ hildon_home_window_applets_init (HildonHomeWindow * window)
   HildonHomeWindowPrivate *priv = window->priv;
   gchar *user_filename = NULL; 
   gchar *filename = NULL;
+  GError *error = NULL;
 
   user_filename = g_build_filename (g_getenv ("HOME"),
                                     HH_AREA_CONFIGURATION_FILE,
@@ -718,22 +430,137 @@ hildon_home_window_applets_init (HildonHomeWindow * window)
     filename = HH_AREA_GLOBAL_CONFIGURATION_FILE;
   
   hildon_home_area_load_configuration (HILDON_HOME_AREA (priv->applet_area),
-                                       filename);
+                                       filename,
+                                       &error);
+
+  if (error)
+    {
+      g_signal_emit_by_name (window, "io-error", error);
+      g_error_free (error);
+    }
 
   g_free (user_filename);
 }
 
+static void
+hildon_home_window_size_allocate (GtkWidget *widget,
+                                  GtkAllocation *allocation)
+{
+  HildonHomeWindow *window;
+  HildonHomeWindowPrivate *priv;
+  GtkRequisition child_req;
+  GtkAllocation  child_allocation = {0};
+
+  window = HILDON_HOME_WINDOW (widget);
+  priv = window->priv;
+
+  widget->allocation = *allocation;
+
+  if (!priv->work_area)
+    return;
+
+  if (GTK_IS_WIDGET (priv->titlebar))
+    {
+      gtk_widget_size_request (priv->titlebar, &child_req);
+      child_allocation.x = priv->work_area->x;
+      child_allocation.y = priv->work_area->y;
+      child_allocation.width = priv->work_area->width;
+      child_allocation.height = MIN (priv->work_area->height, child_req.height);
+
+      gtk_widget_size_allocate (priv->titlebar, &child_allocation);
+    }
+
+  if (GTK_IS_WIDGET (priv->applet_area))
+    {
+      child_allocation.y += child_allocation.height;
+      child_allocation.height = priv->work_area->height -
+                                child_allocation.height;
+
+      child_allocation.x = priv->work_area->x;
+      child_allocation.width = priv->work_area->width;
+      
+      gtk_widget_size_allocate (priv->applet_area, &child_allocation);
+    }
+}
+
+static gint
+hildon_home_window_expose (GtkWidget       *widget,
+                           GdkEventExpose  *event)
+{
+  HildonHomeWindow *window;
+  HildonHomeWindowPrivate *priv;
+
+  window = HILDON_HOME_WINDOW (widget);
+  priv = window->priv;
+
+  if (GTK_IS_WIDGET (priv->applet_area))
+    gtk_container_propagate_expose (GTK_CONTAINER (widget),
+                                    priv->applet_area,
+                                    event);
+  
+  if (GTK_IS_WIDGET (priv->titlebar))
+    gtk_container_propagate_expose (GTK_CONTAINER (widget),
+                                    priv->titlebar,
+                                    event);
+
+  if (GTK_WIDGET_CLASS (hildon_home_window_parent_class)->expose_event)
+    return GTK_WIDGET_CLASS (hildon_home_window_parent_class)->expose_event (widget,
+                                                                      event);
+  else
+    return TRUE;
+  
+}
+
+static void
+hildon_home_window_map (GtkWidget *widget)
+{
+  HildonHomeWindow *window;
+  HildonHomeWindowPrivate *priv;
+
+  window = HILDON_HOME_WINDOW (widget);
+  priv = window->priv;
+
+  if (GTK_WIDGET_CLASS (hildon_home_window_parent_class)->map)
+    GTK_WIDGET_CLASS (hildon_home_window_parent_class)->map (widget);
+
+  if (GTK_IS_WIDGET (priv->applet_area))
+    gtk_widget_map (priv->applet_area);
+  
+  if (GTK_IS_WIDGET (priv->titlebar))
+    gtk_widget_map (priv->titlebar);
+
+}
+
+
+
+static void
+hildon_home_window_forall (GtkContainer     *container,
+                           gboolean          include_internals,
+                           GtkCallback       callback,
+                           gpointer          callback_data)
+{
+  HildonHomeWindow *window;
+  HildonHomeWindowPrivate *priv;
+
+  window = HILDON_HOME_WINDOW (container);
+  priv = window->priv;
+
+  if (GTK_IS_WIDGET (priv->titlebar) && include_internals)
+    (* callback) (priv->titlebar, callback_data);
+
+  if (GTK_IS_WIDGET (priv->applet_area))
+    (* callback) (priv->applet_area, callback_data);
+}
+
 static GObject *
 hildon_home_window_constructor (GType                  gtype,
-				guint                  n_params,
-				GObjectConstructParam *params)
+                                guint                  n_params,
+                                GObjectConstructParam *params)
 {
   GObject *retval;
   HildonHomeWindow *window;
   HildonHomeWindowPrivate *priv;
   GtkWidget *widget;
-  GtkWidget *vbox;
-  GtkWidget *hbox;
 
   retval = G_OBJECT_CLASS (hildon_home_window_parent_class)->constructor (gtype,
 		  							  n_params,
@@ -742,63 +569,32 @@ hildon_home_window_constructor (GType                  gtype,
   window = HILDON_HOME_WINDOW (retval);
   priv = window->priv;
 
-  gtk_widget_set_app_paintable (widget, TRUE);
-
   gtk_widget_push_composite_child ();
-  
-  /* we push everything inside an alignment because we don't want
-   * to be hidden by the task navigator
-   */
-  priv->align = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
-  gtk_alignment_set_padding (GTK_ALIGNMENT (priv->align),
-		  	     0, 0, /* top, bottom */
-			     HILDON_HOME_LEFT_PADDING, 0);
-  gtk_widget_set_composite_name (priv->align, "hildon-home-main-alignment");
-  gtk_container_add (GTK_CONTAINER (window), priv->align);
-  gtk_widget_show (priv->align);
-  
-  vbox = gtk_vbox_new (FALSE, 0);
-  gtk_widget_set_composite_name (vbox, "hildon-home-main-vbox");
-  gtk_container_add (GTK_CONTAINER (priv->align), vbox);
-  gtk_widget_show (vbox);
-
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_widget_set_size_request (hbox, -1, HILDON_HOME_TOP_BAR_HEIGHT);
-  gtk_widget_set_composite_name (hbox, "hildon-home-top-bar");
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
 
   priv->titlebar = hildon_home_titlebar_new ();
   gtk_widget_set_composite_name (priv->titlebar, "hildon-home-titlebar");
-  g_signal_connect (priv->titlebar, "select-applets-activate",
-		    G_CALLBACK (titlebar_select_applets_activate_cb),
-		    window);
-  g_signal_connect (priv->titlebar, "layout-mode-activate",
-		    G_CALLBACK (titlebar_layout_mode_activate_cb),
-		    window);
-  g_signal_connect (priv->titlebar, "set-background-activate",
-                    G_CALLBACK (titlebar_set_background_activate_cb),
-                    window);
-  g_signal_connect (priv->titlebar, "layout-accept",
-		    G_CALLBACK (titlebar_layout_accept_cb),
-		    window);
-  g_signal_connect (priv->titlebar, "layout-cancel",
-		    G_CALLBACK (titlebar_layout_cancel_cb),
-		    window);
-  gtk_box_pack_start (GTK_BOX (hbox), priv->titlebar, TRUE, TRUE, 0);
+  g_signal_connect_swapped (priv->titlebar, "layout-accept",
+                            G_CALLBACK (hildon_home_window_accept_layout),
+                            window);
+  g_signal_connect_swapped (priv->titlebar, "layout-cancel",
+                            G_CALLBACK (hildon_home_window_cancel_layout),
+                            window);
+  g_object_ref (priv->titlebar);
+  gtk_object_sink (GTK_OBJECT (priv->titlebar));
+  gtk_widget_set_parent (priv->titlebar, widget);
   gtk_widget_show (priv->titlebar);
 
-  priv->main_area = gtk_event_box_new ();
-  gtk_widget_set_composite_name (priv->main_area, "hildon-home-main-area");
-  gtk_box_pack_end (GTK_BOX (vbox), priv->main_area, TRUE, TRUE, 0);
-  gtk_widget_show (priv->main_area);
-
   priv->applet_area = hildon_home_area_new ();
-  gtk_container_add (GTK_CONTAINER(priv->main_area), priv->applet_area);
+  
+  g_object_ref (priv->applet_area);
+  gtk_object_sink (GTK_OBJECT (priv->applet_area));
+  gtk_widget_set_parent (priv->applet_area, widget);
+  gtk_widget_show (priv->applet_area);
+
+  GTK_BIN (widget)->child = priv->applet_area;
+
   HILDON_DESKTOP_WINDOW (window)->container = GTK_CONTAINER (priv->applet_area);
-  g_signal_connect (priv->applet_area, "layout-mode-start",
-                    G_CALLBACK (area_layout_mode_start),
-                    window);
+  
   g_signal_connect (priv->applet_area, "layout-mode-started",
                     G_CALLBACK (area_layout_mode_started),
                     window);
@@ -817,39 +613,6 @@ hildon_home_window_constructor (GType                  gtype,
                     window);
 
   gtk_widget_show_all (priv->applet_area);
-
-  /* realize and set the window type hint; then bind the desktop
-   * GdkWindow to the background manager
-   */
-  gtk_widget_realize (widget);
-  gdk_window_set_type_hint (widget->window, GDK_WINDOW_TYPE_HINT_DESKTOP);
-  
-#if 0
-  priv->bg_manager = background_manager_get_default ();
-  
-  g_signal_connect (priv->bg_manager, "changed",
-		    G_CALLBACK (background_manager_changed_cb),
-		    window);
-  g_signal_connect (priv->bg_manager, "preview",
-                    G_CALLBACK (background_manager_changed_cb),
-                    window);
-  g_signal_connect (priv->bg_manager, "load-complete",
-                    G_CALLBACK (background_manager_load_complete_cb),
-                    window);
-  g_signal_connect (priv->bg_manager, "load-cancel",
-                    G_CALLBACK (background_manager_load_cancel_cb),
-                    window);
-  g_signal_connect (priv->bg_manager, "load-error",
-                    G_CALLBACK (background_manager_load_error_cb),
-                    window);
-
-  background_manager_set_components (priv->bg_manager,
-		  		     get_titlebar_image_from_theme (widget),
-				     get_sidebar_image_from_theme (widget),
-                                     FALSE);
-  background_manager_set_desktop (priv->bg_manager, widget->window);
-#endif
-  
   gtk_widget_pop_composite_child ();
   
   return retval;
@@ -858,20 +621,29 @@ hildon_home_window_constructor (GType                  gtype,
 static void
 hildon_home_window_class_init (HildonHomeWindowClass *klass)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  GParamSpec           *pspec;
+  GObjectClass         *gobject_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass       *widget_class = GTK_WIDGET_CLASS (klass);
+  GtkContainerClass    *container_class = GTK_CONTAINER_CLASS (klass);
 
   gobject_class->constructor = hildon_home_window_constructor;
   gobject_class->set_property = hildon_home_window_set_property;
   gobject_class->get_property = hildon_home_window_get_property;
   gobject_class->finalize = hildon_home_window_finalize;
   
-  widget_class->style_set = hildon_home_window_style_set;
   widget_class->key_press_event = hildon_home_window_key_press_event;
+  widget_class->size_allocate = hildon_home_window_size_allocate;
+  widget_class->expose_event = hildon_home_window_expose;
+  widget_class->map = hildon_home_window_map;
+  widget_class->show = hildon_home_window_show;
+
+  container_class->forall = hildon_home_window_forall;
 
   klass->background = hildon_home_window_background;
   klass->lowmem = hildon_home_window_lowmem;
   klass->system_inactivity = hildon_home_window_system_inactivity;
+  klass->layout_mode_accept = hildon_home_window_layout_mode_accept;
+  klass->layout_mode_cancel = hildon_home_window_layout_mode_cancel;
   
   g_signal_new ("background",
                 G_OBJECT_CLASS_TYPE (gobject_class),
@@ -906,12 +678,45 @@ hildon_home_window_class_init (HildonHomeWindowClass *klass)
                 1,
                 G_TYPE_BOOLEAN);
   
+  g_signal_new ("layout-mode-accept",
+                G_OBJECT_CLASS_TYPE (gobject_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HildonHomeWindowClass, layout_mode_accept),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__VOID,
+                G_TYPE_NONE,
+                0);
+  
+  g_signal_new ("layout-mode-cancel",
+                G_OBJECT_CLASS_TYPE (gobject_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HildonHomeWindowClass, layout_mode_cancel),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__VOID,
+                G_TYPE_NONE,
+                0);
+  
+  g_signal_new ("io-error",
+                G_OBJECT_CLASS_TYPE (gobject_class),
+                G_SIGNAL_RUN_FIRST,
+                G_STRUCT_OFFSET (HildonHomeWindowClass, io_error),
+                NULL,
+                NULL,
+                g_cclosure_marshal_VOID__POINTER,
+                G_TYPE_NONE,
+                1,
+                G_TYPE_POINTER);
+  
+  pspec =  g_param_spec_pointer ("work-area",
+                                 "Work Area",
+                                 "Work area available for desktop",
+                                 G_PARAM_READWRITE);
+
   g_object_class_install_property (gobject_class,
-		                   PROP_OSSO_CONTEXT,
-				   g_param_spec_pointer ("osso-context",
-					   		 "Osso Context",
-							 "Osso context to be used by the window",
-							 (G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+                                   PROP_WORK_AREA,
+                                   pspec);
   
   g_type_class_add_private (klass, sizeof (HildonHomeWindowPrivate));
 }
@@ -921,19 +726,50 @@ hildon_home_window_init (HildonHomeWindow *window)
 {
   HildonHomeWindowPrivate *priv;
   GtkWidget *widget;
+  GdkRectangle work_area;
+  HDWM *wm;
 
-  gtk_window_set_has_frame (GTK_WINDOW (window), FALSE);
 
   widget = GTK_WIDGET (window);
-  gtk_widget_set_size_request (widget,
-                               HILDON_HOME_WINDOW_WIDTH,
-                               HILDON_HOME_WINDOW_HEIGHT);
-
 
   window->priv = priv = HILDON_HOME_WINDOW_GET_PRIVATE (window);
-
-
   priv->is_dimmed = FALSE;
+
+  wm = hd_wm_get_singleton ();
+
+  hd_wm_get_work_area (wm, &work_area);
+  hildon_home_window_set_work_area (window, &work_area);
+
+  g_signal_connect_swapped (wm, "work-area-changed",
+                            G_CALLBACK (hildon_home_window_set_work_area),
+                            window);
+}
+
+static void
+hildon_home_window_set_work_area (HildonHomeWindow *window,
+                                  GdkRectangle *work_area)
+{
+  HildonHomeWindowPrivate   *priv;
+
+  g_return_if_fail (HILDON_IS_HOME_WINDOW (window) && work_area);
+  priv = window->priv;
+
+  if (!priv->work_area || priv->work_area->x != work_area->x
+                       || priv->work_area->y != work_area->y
+                       || priv->work_area->width != work_area->width
+                       || priv->work_area->height != work_area->height)
+    {
+      if (!priv->work_area)
+        priv->work_area = g_new (GdkRectangle, 1);
+
+      priv->work_area->x      = work_area->x;
+      priv->work_area->y      = work_area->y;
+      priv->work_area->width  = work_area->width;
+      priv->work_area->height = work_area->height;
+
+      gtk_widget_queue_resize (GTK_WIDGET (window));
+      g_object_notify (G_OBJECT (window), "work-area");
+    }
 }
 
 GtkWidget *
@@ -951,15 +787,6 @@ hildon_home_window_get_titlebar (HildonHomeWindow *window)
   return window->priv->titlebar;
 }
 
-GtkWidget *
-hildon_home_window_get_main_area (HildonHomeWindow *window)
-{
-  g_return_val_if_fail (HILDON_IS_HOME_WINDOW (window), NULL);
-
-  return window->priv->main_area;
-}
-
-
 void
 hildon_home_window_show_information_note (HildonHomeWindow *window,
                                           const gchar *text)
@@ -975,6 +802,17 @@ hildon_home_window_show_information_note (HildonHomeWindow *window,
     gtk_widget_destroy (GTK_WIDGET (note));
 }
 
+void
+hildon_home_window_show_information_banner (HildonHomeWindow *window,
+                                            const gchar *text)
+{
+  g_return_if_fail (GTK_IS_WIDGET (window) && text);
+
+  hildon_banner_show_information (GTK_WIDGET (window),
+                                  NULL,
+                                  text);
+}
+
 void
 hildon_home_window_set_desktop_dimmed (HildonHomeWindow *window,
                                        gboolean dimmed)
@@ -1030,3 +868,69 @@ hildon_home_window_set_desktop_dimmed (HildonHomeWindow *window,
 
   priv->is_dimmed = dimmed;
 }
+
+void
+hildon_home_window_select_applets (HildonHomeWindow   *window)
+{
+  HildonHomeWindowPrivate *priv = window->priv;
+  
+  priv->selecting_applets = TRUE;
+  g_signal_emit_by_name (window, "select-plugins", NULL);
+  priv->selecting_applets = FALSE;
+
+  if (HILDON_IS_HOME_AREA (priv->applet_area))
+    {
+      HildonHomeArea *area = HILDON_HOME_AREA (priv->applet_area);
+
+      /* If we are in layout mode, we only save when layout mode is
+       * accepted */
+      if (!hildon_home_area_get_layout_mode (area))
+        g_signal_emit_by_name (window, "save", NULL);
+    }
+}
+
+void
+hildon_home_window_layout_mode_activate (HildonHomeWindow   *window)
+{
+  HildonHomeWindowPrivate *priv = window->priv;
+  
+#if 0
+  if (priv->is_lowmem)
+    {
+      hildon_banner_show_information (NULL,
+                                      NULL,
+                                      HH_LOW_MEMORY_TEXT);
+      return;
+    }
+#endif
+
+  hildon_home_area_set_layout_mode (HILDON_HOME_AREA (priv->applet_area),
+                                    TRUE);
+
+}
+
+void
+hildon_home_window_cancel_layout (HildonHomeWindow   *window)
+{
+  g_return_if_fail (window);
+  g_signal_emit_by_name (window, "layout-mode-cancel");
+}
+
+void
+hildon_home_window_accept_layout (HildonHomeWindow   *window)
+{
+  g_return_if_fail (window);
+  g_signal_emit_by_name (window, "layout-mode-accept");
+}
+
+GtkWidget *
+hildon_home_window_get_area (HildonHomeWindow *window)
+{
+  HildonHomeWindowPrivate *priv;
+  
+  g_return_val_if_fail (HILDON_IS_HOME_WINDOW (window), NULL);
+  priv = window->priv;
+
+  return priv->applet_area;
+}
+
