@@ -603,6 +603,8 @@ hd_wm_watched_window_process_hildon_view_list (HDWMWatchedWindow *win)
           HN_DBG("adding view info to AS");
           info = hd_wm_watched_window_view_get_info (new_view);
 
+	  hd_wm_add_applications (hdwm,info);
+
 	  g_signal_emit_by_name (hdwm,"entry_info_added",info);
   
           /* The window may have been 'viewless' before this 
@@ -611,9 +613,12 @@ hd_wm_watched_window_process_hildon_view_list (HDWMWatchedWindow *win)
            */
           if (hd_wm_watched_window_peek_info (win))
             {
+	      gboolean removed_app;
               HN_DBG("adding first view; removing window info from AS");
-            
-	      g_signal_emit_by_name (hdwm,"entry_info_removed",hd_wm_watched_window_peek_info (win)); 
+           
+	      removed_app = hd_wm_remove_applications (hdwm, hd_wm_watched_window_peek_info (win));
+ 
+	      g_signal_emit_by_name (hdwm,"entry_info_removed",removed_app,hd_wm_watched_window_peek_info (win)); 
 
               /*
                * since the window of multiviewed app does not figure in the AS,
@@ -1036,7 +1041,11 @@ hd_wm_watched_window_destroy (HDWMWatchedWindow *win)
     {
       /* only windows of multiwindow apps have their own info */
       HN_DBG("a window of multiwindow application; removing info from AS");
-      g_signal_emit_by_name (hdwm,"entry_info_removed",win->info);
+
+      gboolean removed_app = hd_wm_remove_applications (hdwm,win->info);
+ 
+      g_signal_emit_by_name (hdwm,"entry_info_removed",removed_app,win->info);
+      
       hd_entry_info_free (win->info);
       win->info = NULL;
     }
