@@ -53,7 +53,8 @@
 #include "hd-wm-memory.h"
 #include "hd-entry-info.h"
 #include "osso-manager.h"
-#include "hd-keys.h"
+
+#include "hd-wm-marshalers.h"
 
 #define SAVE_METHOD      "save"
 #define KILL_APPS_METHOD "kill_app"
@@ -62,6 +63,33 @@
 #define TASKNAV_INSENSITIVE_INTERFACE "com.nokia.tasknav.tasknav_insensitive"
 #define TASKNAV_SENSITIVE_INTERFACE "com.nokia.tasknav.tasknav_sensitive"
 
+#define MCE_SERVICE          "com.nokia.mce"
+#define MCE_SIGNAL_INTERFACE "com.nokia.mce.signal"
+#define MCE_SIGNAL_PATH      "/com/nokia/mce/signal"
+
+/* lowmem signals */
+#define LOWMEM_ON_SIGNAL_INTERFACE  "com.nokia.ke_recv.lowmem_on"
+#define LOWMEM_ON_SIGNAL_PATH       "/com/nokia/ke_recv/lowmem_on"
+#define LOWMEM_ON_SIGNAL_NAME       "lowmem_on"
+
+#define LOWMEM_OFF_SIGNAL_INTERFACE "com.nokia.ke_recv.lowmem_off"
+#define LOWMEM_OFF_SIGNAL_PATH      "/com/nokia/ke_recv/lowmem_off"
+#define LOWMEM_OFF_SIGNAL_NAME      "lowmem_off"
+
+/* bgkill signals */
+#define BGKILL_ON_SIGNAL_INTERFACE  "com.nokia.ke_recv.bgkill_on"
+#define BGKILL_ON_SIGNAL_PATH       "/com/nokia/ke_recv/bgkill_on"
+#define BGKILL_ON_SIGNAL_NAME       "bgkill_on"
+
+#define BGKILL_OFF_SIGNAL_INTERFACE "com.nokia.ke_recv.bgkill_off"
+#define BGKILL_OFF_SIGNAL_PATH      "/com/nokia/ke_recv/bgkill_off"
+#define BGKILL_OFF_SIGNAL_NAME      "bgkill_off"
+
+
+/* hardware signals */
+#define HOME_LONG_PRESS "sig_home_key_pressed_long_ind"
+#define HOME_PRESS      "sig_home_key_pressed_ind"
+
 #define LAUNCH_SUCCESS_TIMEOUT 20
 
 #define HD_WM_GET_PRIVATE(o) \
@@ -73,6 +101,7 @@ enum
   HDWM_ENTRY_INFO_ADDED_SIGNAL,
   HDWM_ENTRY_INFO_REMOVED_SIGNAL,
   HDWM_ENTRY_INFO_STACK_CHANGED_SIGNAL,
+  HDWM_WORK_AREA_CHANGED_SIGNAL,
   HDWM_SHOW_A_MENU_SIGNAL,
   HDWM_SIGNALS
 };
@@ -168,6 +197,8 @@ struct _HDWMPrivate   /* Our main struct */
   gboolean      has_focus;
   guint         dnotify_timeout_id;
   gboolean      modal_windows;
+
+  GList		*applications;
 };
 
 static HDWMPrivate *hdwmpriv = NULL; 			/* Singleton instance */
@@ -203,6 +234,7 @@ hd_wm_atoms_init (HDWM *hdwm)
     "_NET_SHOWING_DESKTOP",
     "_NET_WM_PID",
     "_NET_ACTIVE_WINDOW",
+    "_NET_WORKAREA",
     "_NET_CLIENT_LIST",
     "_NET_WM_ICON",
     "_NET_WM_USER_TIME",
@@ -403,15 +435,15 @@ hd_wm_dbus_method_call_handler (DBusConnection *connection,
 }
 
 static DBusHandlerResult
-hd_wm_dbus_signal_handler(DBusConnection *conn, DBusMessage *msg, void *data)
+hd_wm_dbus_signal_handler (DBusConnection *conn, DBusMessage *msg, void *data)
 {
   if (dbus_message_is_signal(msg, MAEMO_LAUNCHER_SIGNAL_IFACE,
 				  APP_DIED_SIGNAL_NAME))
   {
     DBusError err;
     gchar *filename;
-    int pid;
-    int status;
+    gint pid;
+    gint status;
     HDWMWatchableApp *app;
 
 
@@ -453,6 +485,107 @@ hd_wm_dbus_signal_handler(DBusConnection *conn, DBusMessage *msg, void *data)
   return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
+static DBusHandlerResult
+lowmem_handler (DBusConnection *conn,
+                DBusMessage    *msg,
+                void           *data)
+{
+  const gchar *member;
+  HDWM *hdwm = HD_WM (data);
+  
+  if (dbus_message_get_type (msg) != DBUS_MESSAGE_TYPE_SIGNAL)
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  
+  member = dbus_message_get_member (msg);
+  if (!member || member[0] == '\0')
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+  if (strcmp (LOWMEM_ON_SIGNAL_NAME, member) == 0)
+  {
+    hd_wm_memory_lowmem_func (TRUE);
+    g_signal_emit_by_name (hdwm,"entry_info_changed");
+    
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }
+  
+  if (strcmp (LOWMEM_OFF_SIGNAL_NAME, member) == 0)
+  {
+    hd_wm_memory_lowmem_func (FALSE);
+    g_signal_emit_by_name (hdwm,"entry_info_changed");
+	  
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }
+  
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static DBusHandlerResult
+bgkill_handler (DBusConnection *conn,
+                DBusMessage    *msg,
+                void           *data)
+{
+  const gchar *member;
+  HDWM *hdwm = HD_WM (data);
+
+  if (dbus_message_get_type (msg) != DBUS_MESSAGE_TYPE_SIGNAL)
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  
+  member = dbus_message_get_member (msg);
+  if (!member || member[0] == '\0')
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+  if (strcmp (BGKILL_ON_SIGNAL_NAME, member) == 0)
+  {
+    hd_wm_memory_bgkill_func (TRUE);
+    g_signal_emit_by_name (hdwm,"entry_info_changed");
+      
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }
+  
+  if (strcmp (BGKILL_OFF_SIGNAL_NAME, member) == 0)
+  {
+    hd_wm_memory_bgkill_func (FALSE);
+    g_signal_emit_by_name (hdwm,"entry_info_changed");
+      
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }
+  
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+static DBusHandlerResult
+mce_handler (DBusConnection *conn,
+             DBusMessage    *msg,
+             void           *data)
+{
+  const gchar *member;
+
+  g_debug ("processing MCE");
+  
+  if (dbus_message_get_type (msg) != DBUS_MESSAGE_TYPE_SIGNAL)
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  
+  member = dbus_message_get_member (msg);
+  if (!member || member[0] == '\0')
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+  if (strcmp (HOME_LONG_PRESS, member) == 0 && !hd_wm_modal_windows_present())
+  {
+    hd_wm_toggle_desktop ();
+      
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }
+  
+  if (strcmp (HOME_PRESS, member) == 0 && !hd_wm_modal_windows_present())
+  {
+    hd_wm_activate (HD_TN_ACTIVATE_MAIN_MENU);
+
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+  }  
+ 
+  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
 static int
 hd_wm_osso_kill_method (GArray *arguments, gpointer data)
 {
@@ -473,18 +606,14 @@ hd_wm_osso_kill_method (GArray *arguments, gpointer data)
     return 1;
 
   if (strcmp(operation, "lru") == 0)
-    {
-      return hd_wm_memory_kill_lru();
-    }
-  else if (strcmp(operation, "all") == 0)
-    {
-      return hd_wm_memory_kill_all_watched(TRUE);
-    }
-  else if (strcmp(operation, "app") != 0 || (arguments->len < 2) )
-    {
-      return 1;
-    }
-
+    return hd_wm_memory_kill_lru ();
+  else 
+  if (strcmp(operation, "all") == 0)
+    return hd_wm_memory_kill_all_watched (TRUE);
+  else
+  if (strcmp(operation, "app") != 0 || (arguments->len < 2) )
+    return 1;
+  
   /* Kill a certain application */
 
   appname = (gchar *)g_array_index(arguments, osso_rpc_t, 1).value.s;
@@ -535,8 +664,8 @@ hd_wm_class_init (HDWMClass *hdwm_class)
 		     G_SIGNAL_RUN_LAST,
 		     G_STRUCT_OFFSET (HDWMClass,entry_info_removed),
 		     NULL, NULL,
-		     g_cclosure_marshal_VOID__POINTER,
-		     G_TYPE_NONE, 1, G_TYPE_POINTER);
+		     g_cclosure_user_marshal_VOID__BOOLEAN_POINTER,
+		     G_TYPE_NONE, 2, G_TYPE_BOOLEAN, G_TYPE_POINTER);
 
   hdwm_signals[HDWM_ENTRY_INFO_STACK_CHANGED_SIGNAL] = 
 	g_signal_new("entry_info_stack_changed",
@@ -546,6 +675,17 @@ hd_wm_class_init (HDWMClass *hdwm_class)
 		     NULL, NULL,
 		     g_cclosure_marshal_VOID__POINTER,
 		     G_TYPE_NONE, 1, G_TYPE_POINTER);
+  
+  hdwm_signals[HDWM_WORK_AREA_CHANGED_SIGNAL] = 
+	g_signal_new("work-area-changed",
+		     G_OBJECT_CLASS_TYPE(object_class),
+		     G_SIGNAL_RUN_FIRST,
+		     G_STRUCT_OFFSET (HDWMClass, work_area_changed),
+		     NULL, NULL,
+		     g_cclosure_marshal_VOID__POINTER,
+		     G_TYPE_NONE,
+             	     1,
+             	     G_TYPE_POINTER);
 
   hdwm_signals[HDWM_SHOW_A_MENU_SIGNAL] = 
 	g_signal_new("show_menu",
@@ -558,22 +698,49 @@ hd_wm_class_init (HDWMClass *hdwm_class)
 
 }
 
+static void hd_wm_register_object_path (HDWM *hdwm,
+					DBusConnection *conn,
+					DBusObjectPathMessageFunction func,
+					gchar *interface,
+					gchar *path)
+{
+  DBusObjectPathVTable vtable;
+  gchar *match_rule;
+  gboolean res;
+
+  vtable.message_function    = func;
+  vtable.unregister_function = NULL;
+
+  match_rule = g_strdup_printf("type='signal', interface='%s'", interface);
+
+  res = dbus_connection_register_object_path (conn, path, &vtable, hdwm);
+
+  if (res)
+  { 
+    g_debug ("%s Registered",interface);
+    dbus_bus_add_match (conn, match_rule, NULL);
+    dbus_connection_flush(conn);
+  }
+  else
+    g_debug ("I couldn't register %s",interface);
+    
+  g_free (match_rule);
+}			
+
 static void 
 hd_wm_init (HDWM *hdwm)
 {
-  DBusConnection *connection;
-  DBusError       error;
+  DBusConnection *connection,*sys_connection;
+  DBusError       error,sys_error;
   gchar          *match_rule = NULL;
   GdkKeymap      *keymap;
-	
+  
   hdwm->priv = hdwmpriv = HD_WM_GET_PRIVATE (hdwm);
 	
-  /*memset(&(hdwm->priv), 0, sizeof(HDWMPrivate));*/
+  memset(hdwm->priv, 0, sizeof(HDWMPrivate));
   
   osso_manager_t *osso_man = osso_manager_singleton_get_instance();
 
-  /*hdwm->priv->app_switcher = as;*/
-
   /* Check for configurable lowmem values. */
 
   hdwm->priv->lowmem_min_distance
@@ -595,14 +762,11 @@ hd_wm_init (HDWM *hdwm)
 #if 0
   application_switcher_set_dnotify_handler (as, &hd_wm_dnotify_func);
   application_switcher_set_shutdown_handler (as, &hd_wm_shutdown_func);
-  application_switcher_set_lowmem_handler (as, &hd_wm_memory_lowmem_func);
-  application_switcher_set_bgkill_handler (as, &hd_wm_memory_bgkill_func);
-
-  g_signal_connect (as, "lowmem",   G_CALLBACK (hd_wm_lowmem_cb),   NULL);
-  g_signal_connect (as, "bgkill",   G_CALLBACK (hd_wm_bgkill_cb),   NULL);
 #endif
   /* build our hash of watchable apps via .desktop key/values */
 
+  hdwm->priv->applications = NULL;
+  
   hdwm->priv->watched_apps = hd_wm_watchable_apps_init ();
 
   /* Initialize the common X atoms */
@@ -642,31 +806,32 @@ hd_wm_init (HDWM *hdwm)
 
   /* Setup shortcuts */
 
-  hdwm->priv->keys = hd_keys_init ();
+  hdwm->keys = hd_keys_config_get_singleton ();
 
   /* Track changes in the keymap */
 
   keymap = gdk_keymap_get_default ();
-  g_signal_connect (G_OBJECT (keymap), "keys-changed",
-                    G_CALLBACK (hd_keys_reload), hdwm->priv->keys); 
+  g_signal_connect (G_OBJECT (keymap), 
+		    "keys-changed",
+		    G_CALLBACK (hd_keys_reload),
+		    hdwm->keys);
 
   /* Get on the DBus */
 
   dbus_error_init (&error);
+  dbus_error_init (&sys_error);
 
-  connection = dbus_bus_get( DBUS_BUS_SESSION, &error );
-
+  connection     = dbus_bus_get (DBUS_BUS_SESSION, &error);
+  sys_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &sys_error);
+  
   if (!connection)
   {
-      osso_log(LOG_ERR, "Failed to connect to DBUS: %s!\n", error.message );
+      g_debug ("Failed to connect to DBUS: %s!\n", error.message );
       dbus_error_free( &error );
   }
   else
   {
-    /* Match rule */
-
-    match_rule = g_strdup_printf("interface='%s'",
-				   APP_LAUNCH_BANNER_METHOD_INTERFACE );
+    match_rule = g_strdup_printf("interface='%s'", APP_LAUNCH_BANNER_METHOD_INTERFACE );
 
     dbus_bus_add_match( connection, match_rule, NULL );
     g_free (match_rule);
@@ -674,29 +839,65 @@ hd_wm_init (HDWM *hdwm)
     match_rule = g_strdup_printf("type='signal', interface='%s'",APPKILLER_SIGNAL_INTERFACE);
       
     dbus_bus_add_match( connection, match_rule, NULL );
-    dbus_connection_add_filter(connection, hd_wm_dbus_signal_handler, NULL, NULL);
+    dbus_connection_add_filter (connection, hd_wm_dbus_signal_handler, hdwm, NULL);
     g_free(match_rule);
 
     match_rule = g_strdup_printf("interface='%s'", TASKNAV_INSENSITIVE_INTERFACE );
 
-    dbus_bus_add_match( connection, match_rule, NULL );
+    dbus_bus_add_match (connection, match_rule, NULL );
 
-    dbus_connection_add_filter( connection, hd_wm_dbus_method_call_handler,/*model */ hdwm, NULL );
+    dbus_connection_add_filter (connection, hd_wm_dbus_method_call_handler, hdwm, NULL );
     g_free(match_rule);
 
-      /* Setup match rule for Maemo Launcher */
     match_rule = g_strdup_printf("type='signal', interface='%s'", MAEMO_LAUNCHER_SIGNAL_IFACE);
 
-    dbus_bus_add_match(connection, match_rule, NULL);
-    dbus_connection_add_filter(connection, hd_wm_dbus_signal_handler,
-				 NULL, NULL);
+    dbus_bus_add_match (connection, match_rule, NULL);
+    dbus_connection_add_filter (connection, hd_wm_dbus_signal_handler, hdwm, NULL);
     g_free(match_rule);
 
     dbus_connection_flush(connection);
 
-      /* Add the com.nokia.tasknav callbacks - FIXME: what are these for ? */
-    add_method_cb(osso_man, KILL_APPS_METHOD, &hd_wm_osso_kill_method, osso_man);
+    /* Add the com.nokia.tasknav callbacks - FIXME: what are these for ? */
 
+    add_method_cb (osso_man, KILL_APPS_METHOD, &hd_wm_osso_kill_method, osso_man);
+  }
+
+  if (!sys_connection)
+  {
+      g_debug ("Failed to connect to DBUS: %s!\n", sys_error.message );
+      dbus_error_free( &sys_error );
+  }
+  else
+  {
+    hd_wm_register_object_path (hdwm, 
+		    		sys_connection,
+				mce_handler,
+				MCE_SIGNAL_INTERFACE,
+				MCE_SIGNAL_PATH); 
+    
+    hd_wm_register_object_path (hdwm,
+		    		sys_connection,
+				bgkill_handler,
+				BGKILL_ON_SIGNAL_INTERFACE,
+				BGKILL_ON_SIGNAL_PATH);
+
+    hd_wm_register_object_path (hdwm,
+		    		sys_connection,
+				bgkill_handler,
+				BGKILL_OFF_SIGNAL_INTERFACE,
+				BGKILL_OFF_SIGNAL_PATH);
+    
+    hd_wm_register_object_path (hdwm,
+		    		sys_connection,
+				lowmem_handler,
+				LOWMEM_ON_SIGNAL_INTERFACE,
+				LOWMEM_ON_SIGNAL_PATH);
+
+    hd_wm_register_object_path (hdwm,
+		    		sys_connection,
+				lowmem_handler,
+				LOWMEM_OFF_SIGNAL_INTERFACE,
+				LOWMEM_OFF_SIGNAL_PATH);
   }
 }
 
@@ -1418,6 +1619,185 @@ client_list_steal_foreach_func (gpointer key,
   return TRUE;
 }
 
+static HDEntryInfo *
+hd_wm_find_app_for_child (HDWM *hdwm, HDEntryInfo *entry_info)
+{
+  GList * l = hdwm->priv->applications;
+  HDWMWatchableApp *app = hd_entry_info_get_app(entry_info);
+        
+  while (l)
+  {
+    HDEntryInfo *e = (HDEntryInfo *)l->data;
+            
+    if (app == hd_entry_info_get_app(e))
+      return e;
+    l = g_list_next(l);
+  }
+
+  return NULL;
+}
+
+void 
+hd_wm_close_application (HDWM *hdwm, HDEntryInfo *entry_info)
+{
+  HDWMWatchedWindow *appwindow;
+  const GList *children = NULL, *l;
+	
+  if (!entry_info)/* || entry_info->type != HD_ENTRY_WATCHED_WINDOW)*/
+  {
+    g_warning ("%s: Tried to close not an application",__FILE__);
+    return;
+  }
+
+  children = hd_entry_info_get_children (entry_info);
+
+  for (l = children ; l ; l = g_list_next (l))
+  {
+    appwindow = hd_entry_info_get_window ((HDEntryInfo*)l->data);
+
+    hd_wm_watched_window_close (appwindow);
+  }
+
+  appwindow = hd_entry_info_get_window (entry_info);
+
+  hd_wm_watched_window_close (appwindow);
+}
+
+void 
+hd_wm_add_applications (HDWM *hdwm, HDEntryInfo *entry_info)
+{
+  HDWMWatchableApp *app;
+  HDEntryInfo	   *e;
+
+  if (!entry_info)
+  {
+    g_warning ("No entry info provided!");
+    return;
+  }
+  
+  switch(entry_info->type)
+  {
+    case HD_ENTRY_WATCHED_WINDOW:
+    case HD_ENTRY_WATCHED_VIEW:
+      /*
+       * because initial windows get created before we have a chance to add
+       * the application item, we have to store orphan windows in temporary
+       * list and process them when the application item is added
+       */
+      app = hd_entry_info_get_app (entry_info);
+      e   = hd_wm_find_app_for_child (hdwm,entry_info);
+
+      if (!e)
+      {
+        e = hd_wm_watchable_app_get_info(app);
+        if (!e)
+        {
+          g_warning ("Could not create HDEntryInfo for app.");
+          return;
+        }
+        
+	hdwm->priv->applications = g_list_prepend (hdwm->priv->applications, e);
+      }
+
+      g_debug ("add_child: %p %p",e,entry_info);        
+
+      hd_entry_info_add_child (e, entry_info);
+
+      break;
+      
+    case HD_ENTRY_WATCHED_APP:
+      /* we handle adding of applications internally in AS */
+      g_warning ("asked to append HD_ENTRY_WATCHED_APP "
+                 "-- this should not happen");
+      return;
+      
+    default:
+      g_warning ("Unknown info type");
+      return;
+  }
+
+}
+
+gboolean  
+hd_wm_remove_applications (HDWM *hdwm, HDEntryInfo *entry_info)
+{
+  HDEntryInfo * info_parent = NULL;
+  gboolean removed_app = FALSE;
+	
+  switch (entry_info->type)
+  {
+    case HD_ENTRY_WATCHED_WINDOW:
+    case HD_ENTRY_WATCHED_VIEW:
+      g_debug ("removing child from AS ...");
+      info_parent = hd_entry_info_get_parent(entry_info);
+
+      if (!info_parent)
+      {
+        g_warning("An orphan HDEntryInfo !!!");
+        return FALSE;
+      }
+
+      if (!hd_entry_info_remove_child (info_parent, entry_info))
+      {
+        g_debug ("... no more children, removing app.");
+        hdwm->priv->applications = 
+	  g_list_remove (hdwm->priv->applications, info_parent);
+	
+        removed_app = TRUE;
+      }
+      break;
+      
+    case HD_ENTRY_WATCHED_APP:
+      /* we handle adding/removing of applications internally in AS */
+      g_warning("asked to remove HD_ENTRY_WATCHED_APP -- this should not happen");
+      return FALSE;
+      
+    default:
+      g_warning("Unknown info type");
+      return FALSE;
+  }
+
+  if (removed_app)
+  {
+    /* we need to check that not all of the remaining apps are
+     * hibernating, and if they are, wake one of them up, because
+     * we will not receive current window msg from MB
+     */
+    GList * l;
+    gboolean all_asleep = TRUE;
+      
+    for (l = hdwm->priv->applications; l != NULL; l = l->next)
+    {
+      HDEntryInfo * entry = l->data;
+      HDWMWatchableApp * app = hd_entry_info_get_app (entry);
+
+      if (app && !hd_wm_watchable_app_is_hibernating (app))
+      {
+        all_asleep = FALSE;
+        break;
+      }
+    }
+
+    if (all_asleep && hdwm->priv->applications)
+    {
+      /*
+       * Unfortunately, we do not know which application is the
+       * most recently used one, so we just wake up the first one
+       */
+       hd_wm_top_item ((HDEntryInfo*)hdwm->priv->applications->data);
+    }
+  }
+ 
+  return removed_app;
+}
+
+GList * 
+hd_wm_get_applications (HDWM *hdwm)
+{
+  return hdwm->priv->applications;
+}
+
+
 static void
 hd_wm_process_x_client_list (HDWM *hdwm)
 {
@@ -1510,7 +1890,8 @@ hd_wm_process_x_client_list (HDWM *hdwm)
 	      if (!info)
                 {
 		  info  = hd_wm_watched_window_create_new_info (win);
-		  g_debug  ("Adding AS entry for view-less window\n");
+		  g_debug ("Adding AS entry for view-less window\n");
+		  hd_wm_add_applications (hdwm,info);
 		  g_signal_emit_by_name (hdwm,"entry_info_added",info);		
 	        }
 	    }
@@ -1522,16 +1903,13 @@ hd_wm_process_x_client_list (HDWM *hdwm)
 }
 
 static void 
-hd_wm_set_window_focus (GtkWindow *window, gboolean focus)
+hd_wm_set_window_focus (GdkWindow *window, gboolean focus)
 {
-  g_assert (window && GTK_IS_WINDOW (window));
-	
-  gtk_window_set_accept_focus (window, focus);
+  g_assert (window && GDK_IS_WINDOW (window));
 
-  if (focus)
-    gtk_window_present   (window);
-  else
-    gtk_window_set_focus (window, NULL);
+  gdk_window_focus (window,GDK_CURRENT_TIME);
+  gdk_window_raise (window);
+  gdk_window_set_accept_focus (window,focus);
 }
 
 void 
@@ -1543,6 +1921,12 @@ hd_wm_set_others_menu_button (HDWM *hdwm, GtkWidget *widget)
   hdwm->priv->others_menu = widget;
 }
 
+void 
+hd_wm_update_client_list (HDWM *hdwm)
+{
+  hd_wm_process_x_client_list (hdwm);
+}
+
 gboolean
 hd_wm_add_watched_window (HDWMWatchedWindow *win)
 {
@@ -1636,7 +2020,7 @@ hd_wm_focus_active_window (HDWM *hdwm)
 }
 
 void
-hd_wm_activate_window (guint32 what, GtkWindow *window)
+hd_wm_activate_window (guint32 what, GdkWindow *window)
 {
   /*GtkWidget * button = NULL;*/
   HDWM *hdwm = hd_wm_get_singleton ();
@@ -1686,7 +2070,7 @@ hdwm_power_key_timeout (gpointer data)
   if (hdwm->priv->shortcut != NULL &&
       hdwm->priv->shortcut->action == HD_KEY_ACTION_POWER)
   {
-    hdwm->priv->shortcut->action_func (hdwm->priv->keys, GINT_TO_POINTER(TRUE));
+    hdwm->priv->shortcut->action_func (hdwm->keys, GINT_TO_POINTER(TRUE));
     hdwm->priv->shortcut = NULL;
   }
 
@@ -1744,7 +2128,7 @@ hd_wm_x_event_filter (GdkXEvent *xevent,
   if (((XEvent*)xevent)->type == KeyPress)
   {
     XKeyEvent *kev = (XKeyEvent *)xevent;
-    hdwm->priv->shortcut = hd_keys_handle_keypress (hdwm->priv->keys, kev->keycode, kev->state); 
+    hdwm->priv->shortcut = hd_keys_handle_keypress (hdwm->keys, kev->keycode, kev->state); 
 
     if (hdwm->priv->shortcut != NULL &&
         hdwm->priv->shortcut->action == HD_KEY_ACTION_POWER &&
@@ -1768,7 +2152,7 @@ hd_wm_x_event_filter (GdkXEvent *xevent,
     if (hdwm->priv->shortcut != NULL)
     {
       if (!hd_wm_modal_windows_present())
-        hdwm->priv->shortcut->action_func (hdwm->priv->keys, hdwm->priv->shortcut->action_func_data);
+        hdwm->priv->shortcut->action_func (hdwm->keys, hdwm->priv->shortcut->action_func_data);
       
       hdwm->priv->shortcut = NULL;
     }
@@ -1790,6 +2174,14 @@ hd_wm_x_event_filter (GdkXEvent *xevent,
     else 
     if (prop->atom == hdwm->priv->atoms[HD_ATOM_MB_CURRENT_APP_WINDOW])
       hd_wm_process_mb_current_app_window (hdwm);
+    else
+    if (prop->atom == hdwm->priv->atoms[HD_ATOM_NET_WORKAREA])
+    {
+      GdkRectangle work_area;
+
+      hd_wm_get_work_area (hdwm, &work_area);
+      g_signal_emit_by_name (hdwm, "work-area-changed", &work_area);
+    }
     else 
     if (prop->atom == hdwm->priv->atoms[HD_ATOM_NET_SHOWING_DESKTOP])
     {
@@ -2261,12 +2653,6 @@ hd_wm_set_bg_kill_situation(gboolean b)
   hdwmpriv->bg_kill_situation = b;
 }
 
-HDAppSwitcher *
-hd_wm_get_app_switcher(void)
-{
-  return NULL;
-}
-
 gint
 hd_wm_get_timer_id(void)
 {
@@ -2411,3 +2797,27 @@ hd_wm_dnotify_register ()
   }
 }
 
+void
+hd_wm_get_work_area (HDWM *hdwm, GdkRectangle *work_area)
+{
+  int *work_area_data;
+
+  work_area_data =
+      hd_wm_util_get_win_prop_data_and_validate (
+                     GDK_WINDOW_XID (gdk_get_default_root_window()),
+                     hdwm->priv->atoms[HD_ATOM_NET_WORKAREA],
+                     XA_CARDINAL,
+                     32,
+                     4,
+                     NULL);
+
+  if (work_area_data)
+    {
+      work_area->x      = work_area_data[0];
+      work_area->y      = work_area_data[1];
+      work_area->width  = work_area_data[2];
+      work_area->height = work_area_data[3];
+
+      XFree (work_area_data);
+    }
+}
