@@ -28,13 +28,11 @@
 
 #include <stdlib.h>
 
-#include <hildon-base-lib/hildon-base-dnotify.h>
-
 #include <glib.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
-#include <libhildondesktop/desktop-panel-window.h>
+#include <hildon-base-lib/hildon-base-dnotify.h>
 #include <libhildondesktop/hildon-desktop-window.h>
 
 #include "hd-desktop.h"
@@ -42,6 +40,7 @@
 #include "hd-config.h"
 #include "hd-plugin-manager.h"
 #include "hd-home-window.h"
+#include "hd-panel-window.h"
 
 #define HD_DESKTOP_GET_PRIVATE(object) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((object), HD_TYPE_DESKTOP, HDDesktopPrivate))
@@ -548,7 +547,7 @@ hd_desktop_load_containers (HDDesktop *desktop)
     }
     else if (!g_ascii_strcasecmp (type, HD_CONTAINER_TYPE_PANEL_BOX))
     {
-      DPanelWinOrientation orientation;
+      HildonDesktopPanelWindowOrientation orientation;
       gchar *orientation_str;
       gint x, y, width, height;
 
@@ -627,18 +626,18 @@ hd_desktop_load_containers (HDDesktop *desktop)
         continue;
       }
 
-      if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_NORTH))
-        orientation = DPANEL_ORIENTATION_TOP; 
-      else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_SOUTH))
-        orientation = DPANEL_ORIENTATION_BOTTOM; 
-      else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_WEST))
-        orientation = DPANEL_ORIENTATION_LEFT; 
-      else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_EAST))
-        orientation = DPANEL_ORIENTATION_RIGHT; 
+      if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_TOP))
+        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_TOP; 
+      else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_BOTTOM))
+        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_BOTTOM; 
+      else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_LEFT))
+        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT; 
+      else if (!g_ascii_strcasecmp (orientation_str, HD_WINDOW_ORIENTATION_RIGHT))
+        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_RIGHT; 
       else 
-        orientation = DPANEL_ORIENTATION_LEFT; 
+        orientation = HILDON_DESKTOP_PANEL_WINDOW_ORIENTATION_LEFT; 
 
-      container = g_object_new (DESKTOP_TYPE_PANEL_WINDOW,
+      container = g_object_new (HD_TYPE_PANEL_WINDOW,
                                "x", x,
                                "y", y,
                                "width", width,
