@@ -22,13 +22,26 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib-object.h>
 
-#include "hd-plugin-loader.h"
-#include "hd-plugin-loader-legacy.h"
 #include "hd-plugin-loader-factory.h"
+#include "hd-plugin-loader.h"
+#include "hd-plugin-loader-default.h"
+#include "hd-plugin-loader-builtin.h"
+#include "hd-plugin-loader-legacy.h"
 #include "hd-config.h"
 
+#ifndef HD_PLUGIN_LOADER_MODULES_PATH
+#define HD_PLUGIN_LOADER_MODULES_PATH "/usr/lib/hildon-desktop/loaders"
+#endif
+
+#define MODULE_LOAD_SYMBOL 	   "hd_plugin_loader_module_type"
+#define MODULE_GET_INSTANCE_SYMBOL "hd_plugin_loader_module_get_instance"
+
 #define HD_PLUGIN_LOADER_FACTORY_GET_PRIVATE(object) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((object), HD_TYPE_PLUGIN_LOADER_FACTORY, HDPluginLoaderFactoryPrivate))
 
@@ -37,14 +50,79 @@ G_DEFINE_TYPE (HDPluginLoaderFactory, hd_plugin_loader_factory, G_TYPE_OBJECT);
 struct _HDPluginLoaderFactoryPrivate 
 {
   GHashTable *registry;
+  GHashTable *modules;
+
+  gchar 	 *(*load_module)   (void);
+  HDPluginLoader *(*get_instance)    (void);
 };
 
+static void 
+hd_plugin_loader_factory_load_modules (HDPluginLoaderFactory *factory)
+{
+  GError *error = NULL;
+  GDir *path_modules;
+  const gchar *name;
+
+  path_modules = g_dir_open (HD_PLUGIN_LOADER_MODULES_PATH, 0, &error);
+
+  if (error != NULL)
+  { 
+    g_warning ("I couldn't open %s path for plugin loaders",
+		HD_PLUGIN_LOADER_MODULES_PATH);
+   
+    g_error_free (error);
+    return;
+  }
+
+  while ((name = g_dir_read_name (path_modules)) != NULL)
+  {
+    if (g_str_has_suffix (name,".so"))
+    {
+      GModule *module;
+      gchar *libpath = g_build_filename (HD_PLUGIN_LOADER_MODULES_PATH, name, NULL);
+
+      module = g_module_open (libpath, G_MODULE_BIND_LAZY);
+
+      if (module != NULL)
+      {
+        if (g_module_symbol (module,
+			     MODULE_LOAD_SYMBOL,
+			     (gpointer *) &factory->priv->load_module))
+	{
+	  g_hash_table_insert (factory->priv->modules,
+			       factory->priv->load_module (),
+			       module);
+	}
+	else
+	{
+	  g_warning (g_module_error ());
+  	  g_module_close (module);
+	}	  
+      }
+      else
+      {
+	g_warning (g_module_error ());
+      }
+    } 
+  }
+   
+  g_dir_close (path_modules); 
+}
+
 static void
 hd_plugin_loader_factory_init (HDPluginLoaderFactory *factory)
 {
   factory->priv = HD_PLUGIN_LOADER_FACTORY_GET_PRIVATE (factory);
 
   factory->priv->registry = g_hash_table_new (g_str_hash, g_str_equal);
+  
+  factory->priv->modules  = 
+    g_hash_table_new_full (g_str_hash, 
+	  		   g_str_equal,
+			   (GDestroyNotify) g_free,
+			   (GDestroyNotify) g_module_close);
+
+  hd_plugin_loader_factory_load_modules (factory);
 }
 
 static gboolean
@@ -56,6 +134,15 @@ hd_plugin_loader_factory_free_loader (gchar *key, GObject *object)
   return TRUE;
 }
 
+static gboolean 
+hd_plugin_loader_factory_free_modules (gchar *key, GModule *module)
+{
+  g_free (key);
+  g_module_close (module);
+
+  return TRUE;
+}
+
 static void
 hd_plugin_loader_factory_finalize (GObject *object)
 {
@@ -72,7 +159,13 @@ hd_plugin_loader_factory_finalize (GObject *object)
                                  (GHRFunc) hd_plugin_loader_factory_free_loader, 
                                  NULL);
 
+    g_hash_table_foreach_remove (priv->registry, 
+                                 (GHRFunc) hd_plugin_loader_factory_free_modules, 
+                                 NULL);
+ 
+
     g_hash_table_destroy (priv->registry);
+    g_hash_table_destroy (priv->modules);
   }
 
   G_OBJECT_CLASS (hd_plugin_loader_factory_parent_class)->finalize (object);
@@ -138,7 +231,7 @@ hd_plugin_loader_factory_create (HDPluginLoaderFactory *factory,
         error->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND)
     {
       /* If no Type= assume old API */
-      type = g_strdup (HD_PLUGIN_LOADER_OLD_API);
+      type = g_strdup (HD_PLUGIN_LOADER_TYPE_OLD_API);
       g_error_free (error);
     }
     else
@@ -148,22 +241,53 @@ hd_plugin_loader_factory_create (HDPluginLoaderFactory *factory,
       return NULL;
     }
   }
+
   loader = (HDPluginLoader *) g_hash_table_lookup (priv->registry, type);
 
   if (!loader) 
   {
-    if (!g_ascii_strcasecmp (type, HD_PLUGIN_LOADER_OLD_API) ||
-        !g_ascii_strcasecmp (type, HD_PLUGIN_LOADER_OLD_API_HOME)) 
+    /* Create instance of plugin loader and add to registry for 
+       later use */
+    if (!g_ascii_strcasecmp (type, HD_PLUGIN_LOADER_TYPE_OLD_API) ||
+        !g_ascii_strcasecmp (type, HD_PLUGIN_LOADER_TYPE_OLD_API_HOME)) 
     {
-      /* Create instance of Old API plugin loader and 
-         add to registry for later use */
       loader = g_object_new (HD_TYPE_PLUGIN_LOADER_LEGACY, NULL);
-      g_hash_table_insert (priv->registry, type, loader);
+    }
+    else if (!g_ascii_strcasecmp (type, HD_PLUGIN_LOADER_TYPE_DEFAULT)) 
+    {
+      loader = g_object_new (HD_TYPE_PLUGIN_LOADER_DEFAULT, NULL);
+    }
+    else if (!g_ascii_strcasecmp (type, HD_PLUGIN_LOADER_TYPE_BUILTIN)) 
+    {
+      loader = g_object_new (HD_TYPE_PLUGIN_LOADER_BUILTIN, NULL);
     }
     else
     {
-      g_warning ("Unknown Plugin Loader type: %s", type);
+      GModule *module = g_hash_table_lookup (priv->modules, type);
+
+      if (module)
+      {
+        if (g_module_symbol (module,
+			     MODULE_GET_INSTANCE_SYMBOL,
+			     (gpointer *) &factory->priv->get_instance))
+        {
+	  loader = priv->get_instance ();
+	  
+	  priv->get_instance = NULL;
+	}
+	else
+	{
+	  g_debug ("%s: module invalid, discarding it for future use",__FILE__);
+	  g_hash_table_remove (priv->modules, type);
+	}
+      }
+      else
+      {
+	g_warning ("Unknown Plugin Loader type: %s", type);
+      }
     }
+
+    g_hash_table_insert (priv->registry, g_strdup (type), loader);
   }
 
   hd_plugin_loader_set_key_file (loader, keyfile);
