@@ -32,4 +32,4 @@ gint   hd_select_plugins_dialog_run  (GList *loaded_plugins,
                                       const gchar *plugin_dir,
                                       GList **selected_plugins);
 
-#endif
+#endif /* __HD_SELECT_PLUGINS_DIALOG_H__ */
